@@ -2,6 +2,7 @@ import json
 import shlex
 import subprocess
 import sys
+from datetime import datetime, timezone
 from enum import Enum
 from logging import Logger
 from pathlib import Path
@@ -33,6 +34,7 @@ from servo.servo import (
 )
 from servo.servo_runner import ServoRunner
 from servo.types import *
+from servo.utilities import PreservedScalarString
 
 # Add the devtools debug() function to the CLI if its available
 try:
@@ -357,7 +359,7 @@ class CLI(typer.Typer):
         ctx.base_url = base_url
 
         # TODO: This should be pluggable
-        if ctx.invoked_subcommand not in {"init", "schema", "generate", "validate"}:
+        if ctx.invoked_subcommand not in {"init", "schema", "generate", "validate", "version"}:
             CLI.assemble_from_context(ctx)
 
     @staticmethod
@@ -384,7 +386,7 @@ class CLI(typer.Typer):
 
         # Assemble the Servo
         try:
-            assembly, servo, ServoSettings = ServoAssembly.assemble(
+            assembly, servo, ServoConfiguration = ServoAssembly.assemble(
                 config_file=ctx.config_file, optimizer=optimizer
             )
         except ValidationError as error:
@@ -406,7 +408,7 @@ class CLI(typer.Typer):
             if isinstance(value, str):
                 # Lookup by key
                 for connector in context.servo.connectors:
-                    if connector.config_key_path == value:
+                    if connector.config_key == value:
                         return connector
                 raise typer.BadParameter(f"no connector found for key '{value}'")
             else:
@@ -414,7 +416,7 @@ class CLI(typer.Typer):
                 for key in value:
                     size = len(connectors)
                     for connector in context.servo.connectors:
-                        if connector.config_key_path == key:
+                        if connector.config_key == key:
                             connectors.append(connector)
                             break
                     if len(connectors) == size:
@@ -465,16 +467,16 @@ class CLI(typer.Typer):
         for key in value:
             if ":" in key:
                 # We have an alias descriptor
-                key_path, identifier = key.split(":", 2)
+                config_key, identifier = key.split(":", 2)
             else:
                 # Vanilla key-path or class name
-                key_path = None
+                config_key = None
                 identifier = key
 
             if connector_class := _connector_class_from_string(identifier):
-                if key_path is None:
-                    key_path = connector_class.__key_path__
-                routes[key_path] = connector_class
+                if config_key is None:
+                    config_key = connector_class.__config_key__
+                routes[config_key] = connector_class
             else:
                 raise typer.BadParameter(f"no connector found for key '{identifier}'")
 
@@ -509,7 +511,7 @@ class ConnectorCLI(CLI):
                     context.connector = connector
 
         if name is None:
-            name = _command_name_from_config_key_path(connector_type.__key_path__)
+            name = _command_name_from_config_key(connector_type.__config_key__)
         if help is None:
             help = connector_type.description
         if isinstance(callback, DefaultPlaceholder):
@@ -597,7 +599,7 @@ class ServoCLI(CLI):
             if customize:
                 check = Check(
                     "Which connectors do you want to activate? ",
-                    choices=list(map(lambda c: c.name, ServoAssembly.all_connectors())),
+                    choices=list(map(lambda c: c.name, ServoAssembly.all_connector_types())),
                     check=" √",
                     margin=2,
                     check_color=colors.bright(colors.foreground["green"]),
@@ -613,8 +615,8 @@ class ServoCLI(CLI):
                     filter(
                         None,
                         map(
-                            lambda c: c.__key_path__ if c.name in result else None,
-                            ServoAssembly.all_connectors(),
+                            lambda c: c.__config_key__ if c.name in result else None,
+                            ServoAssembly.all_connector_types(),
                         ),
                     )
                 )
@@ -641,7 +643,7 @@ class ServoCLI(CLI):
             """
             Display adjustable components
             """
-            results = context.servo.dispatch_event("components")
+            results = context.servo.dispatch_event(Events.COMPONENTS)
             headers = ["COMPONENT", "SETTINGS", "CONNECTOR"]
             table = []
             for result in results:
@@ -683,7 +685,7 @@ class ServoCLI(CLI):
             """
             event_handlers: List[EventHandler] = []
             connectors = (
-                context.assembly.all_connectors() if all else context.servo.connectors
+                context.assembly.all_connector_types() if all else context.assembly.connectors
             )
             for connector in connectors:
                 event_handlers.extend(connector.__event_handlers__)
@@ -827,7 +829,7 @@ class ServoCLI(CLI):
         ) -> None:
             """Manage connectors"""
             connectors = (
-                context.assembly.all_connectors() if all else context.servo.connectors
+                context.assembly.all_connector_types() if all else context.assembly.connectors
             )
             headers = ["NAME", "VERSION", "DESCRIPTION"]
             if verbose:
@@ -863,7 +865,7 @@ class ServoCLI(CLI):
             for connector in connectors:
                 if not connector.responds_to_event(event):
                     raise typer.BadParameter(
-                        f"connectors of type '{connector.__class__.__name__}' do not support checks (at key '{connector.config_key_path}')"
+                        f"connectors of type '{connector.__class__.__name__}' do not support checks (at key '{connector.config_key}')"
                     )
 
         @self.command(section=section)
@@ -884,7 +886,7 @@ class ServoCLI(CLI):
             if connectors:
                 validate_connectors_respond_to_event(connectors, Events.CHECK)
             else:
-                connectors = context.servo.connectors
+                connectors = context.assembly.connectors
 
             results: List[EventResult] = context.servo.dispatch_event(
                 Events.CHECK, include=connectors
@@ -916,7 +918,7 @@ class ServoCLI(CLI):
             if connectors:
                 validate_connectors_respond_to_event(connectors, Events.DESCRIBE)
             else:
-                connectors = context.servo.connectors
+                connectors = context.assembly.connectors
 
             results: List[EventResult] = context.servo.dispatch_event(
                 Events.DESCRIBE, include=connectors
@@ -1056,17 +1058,18 @@ class ServoCLI(CLI):
             _not_yet_implemented()
 
     def add_config_commands(self, section=Section.CONFIG) -> None:
-        class SettingsOutputFormat(AbstractOutputFormat):
+        class ConfigOutputFormat(AbstractOutputFormat):
             yaml = YAML_FORMAT
             json = JSON_FORMAT
             dict = DICT_FORMAT
             text = TEXT_FORMAT
+            configmap = CONFIGMAP_FORMAT
 
         @self.command(section=section)
         def config(
             context: Context,
-            format: SettingsOutputFormat = typer.Option(
-                SettingsOutputFormat.yaml, "--format", "-f", help="Select output format"
+            format: ConfigOutputFormat = typer.Option(
+                ConfigOutputFormat.yaml, "--format", "-f", help="Select output format"
             ),
             output: typer.FileTextWrite = typer.Option(
                 None, "--output", "-o", help="Output configuration to [FILE]"
@@ -1079,24 +1082,56 @@ class ServoCLI(CLI):
             Display configured settings
             """
             include = set(keys) if keys else None
-            settings = context.servo.configuration.dict(
+            config = context.servo.config.dict(
                 exclude_unset=True, include=include
             )
-            settings_json = json.dumps(settings, indent=2, default=pydantic_encoder)
-            settings_dict = json.loads(settings_json)
-            settings_dict_str = pformat(settings_dict)
-            settings_yaml = yaml.dump(settings_dict, indent=2, sort_keys=True)
+            config_json = json.dumps(config, indent=2, default=pydantic_encoder)
+            config_dict = json.loads(config_json)
+            config_dict_str = pformat(config_dict)
+            config_yaml = yaml.dump(config_dict, indent=2, sort_keys=True)
 
-            if format == SettingsOutputFormat.text:
+            if format == ConfigOutputFormat.text:
                 pass
             else:
                 lexer = format.lexer()
-                if format == SettingsOutputFormat.yaml:
-                    data = settings_yaml
-                elif format == SettingsOutputFormat.json:
-                    data = settings_json
-                elif format == SettingsOutputFormat.dict:
-                    data = settings_dict_str
+                if format == ConfigOutputFormat.yaml:
+                    data = config_yaml
+                elif format == ConfigOutputFormat.json:
+                    data = config_json
+                elif format == ConfigOutputFormat.dict:
+                    data = config_dict_str
+                elif format == ConfigOutputFormat.configmap:                    
+                    configured_at = datetime.now(timezone.utc).isoformat()
+                    connectors = []
+                    for connector in context.servo.connectors:
+                        connectors.append({
+                            "name": connector.name,
+                            "description": connector.description,
+                            "version": str(connector.version),
+                            "url": str(connector.homepage),
+                            "config_key": connector.config_key,
+                        })
+                    connectors_json_str = json.dumps(connectors, indent=None)
+
+                    configmap = {
+                        "apiVersion": "v1",
+                        "kind": "ConfigMap",
+                        "metadata": {
+                            "name": "opsani-servo-config",
+                            "labels": {
+                                "app.kubernetes.io/name": "servo",
+                                "app.kubernetes.io/version": str(context.servo.version),
+                            },
+                            "annotations": {
+                                "servo.opsani.com/configured_at": configured_at,
+                                "servo.opsani.com/connectors": connectors_json_str,
+                            }
+                        },
+                        "data": {
+                            "servo.yaml": PreservedScalarString(config_yaml)
+                        }
+                    }
+                    data = yaml.dump(configmap, indent=2, sort_keys=False, explicit_start=True)
                 else:
                     raise RuntimeError(
                         "no handler configured for output format {format}"
@@ -1157,20 +1192,21 @@ class ServoCLI(CLI):
 
                 if connector:
                     if isinstance(connector, Connector):
-                        settings_class = connector.settings.__class__
+                        config_model = connector.config.__class__
                     elif issubclass(connector, Connector):
-                        settings_class = connector.config_model()
+                        config_model = connector.config_model()
                     else:
                         raise typer.BadParameter(
                             f"unexpected connector type '{connector.__class__.__name__}'"
                         )
                 else:
                     CLI.assemble_from_context(context)
-                    settings_class = context.servo.configuration.__class__
+                    config_model = context.servo.config.__class__
+                
                 if format == SchemaOutputFormat.json:
-                    output_data = settings_class.schema_json(indent=2)
+                    output_data = config_model.schema_json(indent=2)
                 elif format == SchemaOutputFormat.dict:
-                    output_data = pformat(settings_class.schema())
+                    output_data = pformat(config_model.schema())
                 else:
                     raise RuntimeError(
                         f"no handler configured for output format {format}"
@@ -1375,11 +1411,11 @@ class ServoCLI(CLI):
             # TODO: Update to work with specific connectors
             if short:
                 if format == VersionOutputFormat.text:
-                    typer.echo(f"{context.servo.name} v{context.servo.version}")
+                    typer.echo(f"{Servo.name} v{Servo.version}")
                 elif format == VersionOutputFormat.json:
                     version_info = {
-                        "name": context.servo.name,
-                        "version": str(context.servo.version),
+                        "name": Servo.name,
+                        "version": str(Servo.version),
                     }
                     typer.echo(json.dumps(version_info, indent=2))
                 else:
@@ -1388,20 +1424,20 @@ class ServoCLI(CLI):
                 if format == VersionOutputFormat.text:
                     typer.echo(
                         (
-                            f"{context.servo.name} v{context.servo.version} ({context.servo.maturity})\n"
-                            f"{context.servo.description}\n"
-                            f"{context.servo.homepage}\n"
-                            f"Licensed under the terms of {context.servo.license}"
+                            f"{Servo.name} v{Servo.version} ({Servo.maturity})\n"
+                            f"{Servo.description}\n"
+                            f"{Servo.homepage}\n"
+                            f"Licensed under the terms of {Servo.license}"
                         )
                     )
                 elif format == VersionOutputFormat.json:
                     version_info = {
-                        "name": context.servo.name,
-                        "version": str(context.servo.version),
-                        "maturity": str(context.servo.maturity),
-                        "description": context.servo.description,
-                        "homepage": context.servo.homepage,
-                        "license": str(context.servo.license),
+                        "name": Servo.name,
+                        "version": str(Servo.version),
+                        "maturity": str(Servo.maturity),
+                        "description": Servo.description,
+                        "homepage": Servo.homepage,
+                        "license": str(Servo.license),
                     }
                     typer.echo(json.dumps(version_info, indent=2))
                 else:
@@ -1417,6 +1453,6 @@ def _run(args: Union[str, List[str]], **kwargs) -> None:
         sys.exit(process.returncode)
 
 
-def _command_name_from_config_key_path(key_path: str) -> str:
+def _command_name_from_config_key(config_key: str) -> str:
     # foo.bar.this_key => this-key
-    return key_path.split(".", 1)[-1].replace("_", "-").lower()
+    return config_key.split(".", 1)[-1].replace("_", "-").lower()
